{# -*- coding: utf-8 -*-

  This file is part of Invenio.
  Copyright (C) 2015-2018 CERN.

  Invenio is free software; you can redistribute it and/or modify it
  under the terms of the MIT License; see LICENSE file for more details.
#}
{%- extends config.ACCOUNTS_COVER_TEMPLATE %}

{% from "invenio_accounts/_macros.html" import render_field, form_errors %}

{% block page_body %}
{%- with form = send_confirmation_form %}
<div class="col-md-6 col-md-offset-3">
  <h1 class="text-center">{{config.ACCOUNTS_SITENAME}}</h1>
  <div class="panel panel-default">
    <div class="panel-body">
      <h3 class="text-center panel-free-title">{{_('Resend Confirmation Email')}}</h3>
      <p class="text-left">{{_('Enter your email address below and we will send you an email confirmation link.')}}</p>
      <form action="{{ url_for_security('login') }}" method="POST" name="login_user_form">
      {{form.hidden_tag()}}
      {{form_errors(form)}}
      {{ render_field(form.email, icon="glyphicon glyphicon-user", autofocus=True, errormsg=False) }}
      <button type="submit" class="btn btn-primary btn-lg btn-block"><i class="fa fa-sign-in"></i> {{_('Send Confirmation')}}</button>
      </form>
    </div>
  </div>
</div>
{%- endwith %}
{% endblock page_body %}
